import 'package:auto_route/auto_route.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:furman_now/src/routes/index.gr.dart';
import 'package:furman_now/src/screens/home/content.dart';
import 'package:furman_now/src/screens/home/state.dart';
import 'package:furman_now/src/utils/theme.dart';
import 'package:provider/provider.dart';

class HomeScreenProvider extends StatelessWidget {
  const HomeScreenProvider({super.key});

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider(
      create: (context) => HomePageState(),
      child: const HomeScreen(),
    );
  }
}

class HomeScreen extends StatefulWidget {
  const HomeScreen({Key? key}) : super(key: key);

  @override
  State<HomeScreen> createState() => _HomeScreenState();
}

class _HomeScreenState extends State<HomeScreen> {
  final ScrollController _controller = ScrollController();

  double overscrollTopAmount = 0;
  double overscrollBottomAmount = 0;

  @override
  void initState() {
    super.initState();

    _controller.addListener(updateScrollPosition);
  }

  updateScrollPosition() {
    // bottom overscroll
    var bottomOverscroll =
        _controller.position.pixels - _controller.position.maxScrollExtent;
    if (bottomOverscroll > 0) {
      setState(() {
        overscrollBottomAmount = bottomOverscroll;
      });
    } else {
      if (overscrollBottomAmount != 0) {
        setState(() {
          overscrollBottomAmount = 0;
        });
      }
    }

    // top overscroll
    var topOverscroll = -_controller.position.pixels;
    if (topOverscroll > 0) {
      setState(() {
        overscrollTopAmount = topOverscroll;
      });
    } else {
      if (overscrollTopAmount != 0) {
        setState(() {
          overscrollTopAmount = 0;
        });
      }
    }

    handleTopOverscroll(topOverscroll);
  }

  handleTopOverscroll(double overscrollTopAmount) {
    const scrollMessageSensitivity = 20;
    const pageSwitchSensitivity = 60;
    var collapsed = context.read<HomePageState>().collapse;

    if (!collapsed) {
      if (overscrollTopAmount > pageSwitchSensitivity) {
        // switch to student ID page
        context.router.navigate(const StudentIdRoute());
        context.read<HomePageState>().collapse = true;
      } else if (overscrollTopAmount > scrollMessageSensitivity) {
        // show student ID hint
        context.read<HomePageState>().showScrollMessage = true;
      } else {
        // hide student ID hint
        context.read<HomePageState>().showScrollMessage = false;
      }
    }

    if (collapsed && -overscrollTopAmount > pageSwitchSensitivity) {
      // switch back to home page
      context.router.navigate(const HomeRoute());
      context.read<HomePageState>().collapse = false;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: AnnotatedRegion<SystemUiOverlayStyle>(
        value: constructOverlayStyle(),
        child: Container(
          color: const Color(0xffb7acc9),
          child: SafeArea(
            child: Container(
              color: const Color(0xffb7acc9),
              child: Stack(
                fit: StackFit.loose,
                children: [
                  const AutoTabsRouter(
                    routes: [
                      HomeRoute(),
                      StudentIdRoute(),
                    ],
                  ),
                  // overscroll indicator color
                  Align(
                    alignment: Alignment.bottomCenter,
                    child: Container(
                      height: overscrollBottomAmount + 30,
                      color: Colors.grey.shade50,
                    ),
                  ),
                  Consumer<HomePageState>(
                    builder: (context, state, _) =>
                      ClipRect(child: HomeContent(
                        controller: _controller,
                        collapse: state.collapse,
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }
}
